#include "Eigen/Dense"
#include <iostream>
#include <fstream>

template<typename T>
T f(const T &p, const T &x) { return p*x; }

template<typename T>
T dfdp(const T&, const T &x) { return x; }

template<typename T>
T ddfdpp(const T&, const T&) { return 0; }

template<typename T, int M>
T E(const T &p, const Eigen::Matrix<T,M,1> &x, const Eigen::Matrix<T,M,1> &y) {
  T r=0;
  for (auto i=0;i<M;i++) r+=pow(f(p,x(i))-y(i),2);
  return r;
}

template<typename T, int M>
T dEdp(const T &p, const Eigen::Matrix<T,M,1> &x, const Eigen::Matrix<T,M,1> &y) {
  T r=0;
  for (auto i=0;i<M;i++) r+=(f(p,x(i))-y(i))*dfdp(p,x(i));
  return 2*r;
}

template<typename T, int M>
T ddEdpp(const T &p, const Eigen::Matrix<T,M,1> &x, const Eigen::Matrix<T,M,1> &y) {
  T r=0;
  for (auto i=0;i<M;i++)
    r+=dfdp(p,x(i))*dfdp(p,x(i))+(f(p,x(i))-y(i))*ddfdpp(p,x(i));
  return 2*r;
}

template<typename T, int M>
void Calibrate(T& p_l, T& p_u, const Eigen::Matrix<T,M,1> &x, const Eigen::Matrix<T,M,1> &y, const T& eps) {
  while (fabs(p_l-p_u)>eps) {
    std::cout << ".";
    T p_m=(p_u+p_l)/2; 
    T r_m=dEdp(p_m,x,y); 
    if (r_m>0) p_u=p_m;
    else if(r_m<0) p_l=p_m;
    else { p_l=p_u=p_m; break; }
  }
  std::cout << std::endl;
}

int main() {
  const int M=10;
  using T=double;
  using VT=Eigen::Matrix<T,M,1>;
  VT x=VT::Random(), y=VT::Random(); 
  x=x.cwiseProduct(x);
  std::ofstream plot_data("data");
  for (int i=0;i<M;i++)
    plot_data << x(i) << " " << y(i) << std::endl;
  plot_data.close();
  T p=1, p_l=p-10, p_u=p+10;
  std::cout << "p=" << p << "; E=" << E(p,x,y) << "; dEdp=" << dEdp(p,x,y) 
	    << "; ddEdpp=" << ddEdpp(p,x,y) << std::endl;
  Calibrate(p_l,p_u,x,y,1e-7);
  std::cout << "p=" << p_l << "; E=" << E(p_l,x,y) << "; dEdp=" 
	    << dEdp(p_l,x,y) << "; ddEdpp=" << ddEdpp(p_l,x,y) << std::endl;
  return 0;
}
